/* 
  Resolves IPv4 addresses to hostnames and hostnames to IPv4 addresses.

  Copyright (C) 2000 MySQL AB
  Copyright (C) 2002 Dmitry V. Levin <ldv@altlinux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#define RESOLVE_VERSION "2.1"

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <getopt.h>

extern const char *__progname;
static int silent;
static int status;

static struct option long_options[] = {
	{"silent", no_argument, 0, 's'},
	{"help", no_argument, 0, 'h'},
	{"version", no_argument, 0, 'V'},
	{0, 0, 0, 0}
};

static void
print_version (void)
{
	printf ("%s version %s\n", __progname, RESOLVE_VERSION);
}

static void __attribute__ ((__noreturn__)) usage (int rc)
{
	print_version ();
	puts ("\
This software comes with ABSOLUTELY NO WARRANTY. This is free software,\n\
and you are welcome to modify and redistribute it under the GPL license\n\
Get hostname based on IP-address or IP-address based on hostname.\n");
	printf ("Usage: %s [OPTIONS] hostname or IP-address\n", __progname);
	puts ("\n\
  -s, --silent   Be more silent.\n\
  -h, --help     Displays this help and exits.\n\
  -V, --version  Displays version information and exits.\n");

	exit (rc);
}

static void
resolve_addr (const char *name, struct in_addr *inp)
{
	struct hostent *hp;

	if (inp->s_addr == INADDR_BROADCAST)
	{
		puts ("broadcast");
		return;
	}

	if (inp->s_addr == INADDR_ANY)
	{
		puts ("any");
		return;
	}

	if ((hp = gethostbyaddr ((char *) inp, sizeof (*inp), AF_INET)))
	{
		char  **aliases;

		if (silent)
		{
			puts (hp->h_name);
			return;
		}

		printf ("Host name(s) of %s: %s", name, hp->h_name);
		for (aliases = hp->h_aliases; *aliases; ++aliases)
			printf (", %s", *aliases);
		putchar ('\n');
	} else
	{
		status = 2;
		error (EXIT_SUCCESS, 0, "Unable to resolve: %s: %s", name,
		       h_errno ? hstrerror (h_errno) : strerror (errno));
	}
}

static void
resolve_name (const char *name)
{
	struct hostent *hp = gethostbyname (name);

	if (!hp)
	{
		status = 2;
		error (EXIT_SUCCESS, 0, "Unable to resolve: %s: %s", name,
		       h_errno ? hstrerror (h_errno) : strerror (errno));
		return;
	}

	if (silent)
	{
		struct in_addr in;

		memcpy ((char *) &in.s_addr, (char *) *hp->h_addr_list,
			sizeof (in.s_addr));
		puts (inet_ntoa (in));
	} else
	{
		char  **p;

		printf ("IP address(es) of %s:", name);
		for (p = hp->h_addr_list; *p; ++p)
		{
			struct in_addr in;

			memcpy (&in.s_addr, *p, sizeof (in.s_addr));
			printf (" %s", inet_ntoa (in));
		}
		putchar ('\n');
	}
}

int
main (int ac, char **av)
{
	int     c;

	while ((c = getopt_long (ac, av, "?shV", long_options, 0)) != EOF)
	{
		switch (c)
		{
			case 's':
				silent = 1;
				break;
			case 'V':
				print_version ();
				exit (EXIT_SUCCESS);
			case 'h':
				usage (EXIT_SUCCESS);
			default:
				usage (EXIT_FAILURE);
		}
	}
	ac -= optind;
	av += optind;
	if (ac <= 0)
		usage (EXIT_FAILURE);

	while (ac--)
	{
		const char *name = *(av++);
		struct in_addr in;

		if (inet_aton (name, &in))
			resolve_addr (name, &in);
		else
			resolve_name (name);
	}

	return status;
}
